import * as Random from "meteor-random";
import * as DataLoader from "dataloader";
import MongoConnector from "./mongo_connector";
export interface IDataLoader<S, T> {
    clear(id: string): void;
    clearAll(): void;
    load(key: string): any;
}
import { Collection, FindOneOptions, Cursor, InsertOneWriteOpResult, InsertWriteOpResult, UpdateWriteOpResult, DeleteWriteOpResultObject, OptionalId, CollectionInsertOneOptions, WithId, CollectionInsertManyOptions, CommonOptions, Condition, UpdateQuery, UpdateOneOptions, UpdateManyOptions, FilterQuery } from "mongodb";
declare type StrictFilterQuery<T> = {
    [P in keyof T]?: Condition<T[P]>;
} & StringRootQuerySelector<T>;
declare type StringRootQuerySelector<T> = {
    $and?: Array<StrictFilterQuery<T>>;
    $nor?: Array<StrictFilterQuery<T>>;
    $or?: Array<StrictFilterQuery<T>>;
    $text?: {
        $search: string;
        $language?: string;
        $caseSensitive?: boolean;
        $diacraticSensitive?: boolean;
    };
    $where?: string | Function;
    $comment?: string;
};
export interface Options<K, V> {
    batch?: boolean;
    cache?: boolean;
    cacheKeyFn?: (key: any) => any;
    cacheMap?: Map<K, Promise<V>> | "lru";
    clearOnInsert?: boolean;
    clearOnUpdate?: boolean;
    selectorKeyFn?: (key: any) => any;
}
export declare class LruCacheWrapper<K, V> {
    cache: any;
    constructor(cacheSize?: number);
    clear(): void;
    get(key: K): any;
    set(key: K, value: V): any;
    delete(key: K): boolean;
}
declare type Filter<A, T> = A extends {
    strict: boolean;
} ? StrictFilterQuery<T> : FilterQuery<T>;
export declare type Strict = {
    strict: boolean;
};
export default class MongoEntity<TSchema, Strict = {}> {
    connector: MongoConnector;
    random: typeof Random;
    private _collectionName;
    private _collection;
    private _singleLoader;
    private _multiLoader;
    private _insertLoaders;
    private _updateLoaders;
    static DefaultCache: any;
    private _cache;
    get collection(): Collection<TSchema>;
    constructor(connector: MongoConnector, collectionName: string, cache?: any);
    createLoader(selectorFunction: (key: any) => Promise<TSchema>, options?: Options<any, TSchema | TSchema[]>): DataLoader<any, TSchema, any>;
    clearUpdateCaches(selector: any): void;
    clearInsertCaches(selector: any): void;
    assignFilter(object: Object, selector: Object, result: Object, include: 0 | 1): (k: string) => void;
    filter(object: Object, selector: Object): TSchema;
    findOneCached(loader: IDataLoader<string, TSchema>, key: string, selector?: Object): Promise<TSchema>;
    findOneCachedById(id: string, selector?: Object): Promise<TSchema>;
    findManyCached(loader: IDataLoader<string, TSchema[]>, key: string, selector?: Object): Promise<TSchema[]>;
    findAllCached(selector?: Object): Promise<TSchema[]>;
    addCacheToOptions(options: any): any;
    find<T = TSchema>(query: Filter<Strict, TSchema>, options?: FindOneOptions<T extends TSchema ? TSchema : T>): Cursor<T>;
    findOne<T = TSchema>(filter: Filter<Strict, TSchema>, options?: FindOneOptions<T extends TSchema ? TSchema : T>): Promise<T | null>;
    insertOne(docs: OptionalId<TSchema>, options?: CollectionInsertOneOptions): Promise<InsertOneWriteOpResult<WithId<TSchema>>>;
    insertMany(docs: Array<OptionalId<TSchema>>, options?: CollectionInsertManyOptions): Promise<InsertWriteOpResult<WithId<TSchema>>>;
    deleteOne(filter: Filter<Strict, TSchema>, options?: CommonOptions & {
        bypassDocumentValidation?: boolean;
    }): Promise<DeleteWriteOpResultObject>;
    deleteMany(filter: Filter<Strict, TSchema>, options?: CommonOptions): Promise<DeleteWriteOpResultObject>;
    dispose(): Promise<DeleteWriteOpResultObject>;
    updateOne(filter: Filter<Strict, TSchema>, update: UpdateQuery<TSchema> | Partial<TSchema>, options?: UpdateOneOptions): Promise<UpdateWriteOpResult>;
    updateMany(filter: Filter<Strict, TSchema>, update: UpdateQuery<TSchema> | Partial<TSchema>, options?: UpdateManyOptions): Promise<UpdateWriteOpResult>;
}
export {};
