"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LruCacheWrapper = void 0;
var DataLoader = require("dataloader");
var LruCacheWrapper = (function () {
    function LruCacheWrapper(cacheSize) {
        if (cacheSize === void 0) { cacheSize = 500; }
        var Lru = require("lru-cache");
        this.cache = new Lru({ max: cacheSize });
    }
    LruCacheWrapper.prototype.clear = function () {
        this.cache.reset();
    };
    LruCacheWrapper.prototype.get = function (key) {
        return this.cache.get(key) || undefined;
    };
    LruCacheWrapper.prototype.set = function (key, value) {
        return this.cache.set(key, value);
    };
    LruCacheWrapper.prototype.delete = function (key) {
        if (this.cache.has(key)) {
            this.cache.del(key);
            return true;
        }
        return false;
    };
    return LruCacheWrapper;
}());
exports.LruCacheWrapper = LruCacheWrapper;
var MongoEntity = (function () {
    function MongoEntity(connector, collectionName, cache) {
        this.connector = connector;
        this._collectionName = collectionName;
        this._cache = cache ? cache : MongoEntity.DefaultCache;
    }
    Object.defineProperty(MongoEntity.prototype, "collection", {
        get: function () {
            if (!this._collection) {
                this._collection = this.connector.collection(this._collectionName);
            }
            return this._collection;
        },
        enumerable: false,
        configurable: true
    });
    MongoEntity.prototype.createLoader = function (selectorFunction, options) {
        var opts = options;
        if (opts.cacheMap == "lru") {
            opts.cacheMap = new LruCacheWrapper();
        }
        var loader = new DataLoader(function (keys) {
            return Promise.all(keys.map(selectorFunction));
        }, opts);
        if (options) {
            if (options.clearOnInsert) {
                if (!this._insertLoaders) {
                    this._insertLoaders = [];
                }
                this._insertLoaders.push({
                    dataLoader: loader,
                    selectorKeyFn: options.selectorKeyFn,
                });
            }
            if (options.clearOnUpdate) {
                if (!this._updateLoaders) {
                    this._updateLoaders = [];
                }
                if (!options.selectorKeyFn) {
                    throw new Error("You need to provide cache key function to determine when cache needs to be updated");
                }
                this._updateLoaders.push({
                    dataLoader: loader,
                    selectorKeyFn: options.selectorKeyFn,
                });
            }
        }
        return loader;
    };
    MongoEntity.prototype.clearUpdateCaches = function (selector) {
        if (this._updateLoaders) {
            this._updateLoaders.forEach(function (u) {
                var key = u.selectorKeyFn && u.selectorKeyFn(selector);
                if (key) {
                    u.dataLoader.clear(key);
                }
                else {
                    u.dataLoader.clearAll();
                }
            });
        }
    };
    MongoEntity.prototype.clearInsertCaches = function (selector) {
        if (this._insertLoaders) {
            this._insertLoaders.forEach(function (u) {
                var key = u.selectorKeyFn && u.selectorKeyFn(selector);
                if (key) {
                    u.dataLoader.clear(key);
                }
                else {
                    u.dataLoader.clearAll();
                }
            });
        }
    };
    MongoEntity.prototype.assignFilter = function (object, selector, result, include) {
        return function (k) {
            if (selector[k] != include) {
                throw new Error("You cannot combine include and exclude!");
            }
            if (include) {
                if (object[k] !== undefined) {
                    result[k] = object[k];
                }
            }
            else {
                delete result[k];
            }
        };
    };
    MongoEntity.prototype.filter = function (object, selector) {
        var keys = Object.keys(selector);
        if (keys.length == 0) {
            throw new Error("You need to specify the selector!");
        }
        var include = selector[keys[0]];
        var result = include ? {} : Object.assign({}, object);
        var selectorFunction = this.assignFilter(object, selector, result, include);
        keys.forEach(selectorFunction);
        return result;
    };
    MongoEntity.prototype.findOneCached = function (loader, key, selector) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, loader.load(key)];
                    case 1:
                        result = _a.sent();
                        if (result && selector) {
                            return [2, this.filter(result, selector)];
                        }
                        else {
                            return [2, result];
                        }
                        return [2];
                }
            });
        });
    };
    MongoEntity.prototype.findOneCachedById = function (id, selector) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                if (!this._singleLoader) {
                    this._singleLoader = this.createLoader(function (loadId) {
                        return _this.collection.findOne({ _id: loadId });
                    }, this.addCacheToOptions({ clearOnUpdate: true, clearOnInsert: true, selectorKeyFn: function (a) { return a._id; } }));
                }
                return [2, this.findOneCached(this._singleLoader, id, selector)];
            });
        });
    };
    MongoEntity.prototype.findManyCached = function (loader, key, selector) {
        return __awaiter(this, void 0, void 0, function () {
            var result;
            var _this = this;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!selector) return [3, 2];
                        return [4, loader.load(key)];
                    case 1:
                        result = _a.sent();
                        return [2, result.map(function (r) { return _this.filter(r, selector); })];
                    case 2: return [2, loader.load(key)];
                }
            });
        });
    };
    MongoEntity.prototype.findAllCached = function (selector) {
        return __awaiter(this, void 0, void 0, function () {
            var _this = this;
            return __generator(this, function (_a) {
                if (!this._multiLoader) {
                    this._multiLoader = this.createLoader(function () { return _this.collection.find().toArray(); }, this.addCacheToOptions({
                        clearOnInsert: true,
                        clearOnUpdate: true,
                        selectorKeyFn: function (a) { return null; },
                    }));
                }
                return [2, this.findManyCached(this._multiLoader, "ALL", selector)];
            });
        });
    };
    MongoEntity.prototype.addCacheToOptions = function (options) {
        if (options.cacheMap) {
            return options;
        }
        else if (this._cache) {
            options.cacheMap = this._cache;
        }
        else {
            options.cacheMap = "lru";
        }
        return options;
    };
    MongoEntity.prototype.find = function (query, options) {
        return this.collection.find(query, options);
    };
    MongoEntity.prototype.findOne = function (filter, options) {
        return this.collection.findOne(filter, options);
    };
    MongoEntity.prototype.insertOne = function (docs, options) {
        this.clearInsertCaches(docs);
        return this.collection.insertOne(docs, options);
    };
    MongoEntity.prototype.insertMany = function (docs, options) {
        this.clearInsertCaches(docs);
        return this.collection.insertMany(docs, options);
    };
    MongoEntity.prototype.deleteOne = function (filter, options) {
        this.clearInsertCaches(filter);
        return this.collection.deleteOne(filter, options);
    };
    MongoEntity.prototype.deleteMany = function (filter, options) {
        this.clearInsertCaches(filter);
        return this.collection.deleteMany(filter, options);
    };
    MongoEntity.prototype.dispose = function () {
        return this.deleteMany({});
    };
    MongoEntity.prototype.updateOne = function (filter, update, options) {
        this.clearUpdateCaches(filter);
        return this.collection.updateOne(filter, update, options);
    };
    MongoEntity.prototype.updateMany = function (filter, update, options) {
        this.clearUpdateCaches(filter);
        return this.collection.updateMany(filter, update, options);
    };
    return MongoEntity;
}());
exports.default = MongoEntity;
